import base64

from aip import AipImageProcess

from pobaidu.lib.Config import pobaiduConfig

from pobaidu.lib.CommonUtils import get_error_info


class ImageProcess(pobaiduConfig):
    def __init__(self):
        self.BAIDU_AI_CFG = None
        self.CLIENT_API = None
        self.CLIENT_SECRET = None
        self.CLIENT_ID = None
        self.CLIENT = None

    def set_config(self, configPath):
        self.BAIDU_AI_CFG = self.get_config(configPath)
        if self.BAIDU_AI_CFG['baidu-ai']['client_api'] and self.BAIDU_AI_CFG['baidu-ai']['client_secret'] and \
                self.BAIDU_AI_CFG['baidu-ai']['client_id']:
            self.CLIENT_API = self.BAIDU_AI_CFG['baidu-ai']['client_api']
            self.CLIENT_SECRET = self.BAIDU_AI_CFG['baidu-ai']['client_secret']
            self.CLIENT_ID = self.BAIDU_AI_CFG['baidu-ai']['client_id']
            self.CLIENT = AipImageProcess(self.CLIENT_ID, self.CLIENT_API, self.CLIENT_SECRET)

    def get_file_content(self, filePath):
        """
        读取图片
        :param filePath: 读取图片
        :return:
        """
        with open(filePath, 'rb') as fp:
            return fp.read()

    def save_file_content(self, filePath, fileData):
        """
        将图片保存为文件
        :param filePath: 输出位置
        :param fileData: 图片数据，base64格式
        :return:
        """
        if get_error_info(fileData):
            print(get_error_info(fileData))
        else:
            # 解码图片
            imgdata = base64.b64decode(fileData['image'])
            # 将图片保存为文件
            with open(filePath, 'wb') as f:
                f.write(imgdata)

    def selfieAnime(self, filePath):
        """
        人像动漫化
        :param filePath:
        :return:
        """
        file_content = self.get_file_content(filePath)
        res_image = self.CLIENT.selfieAnime(file_content)
        return res_image

    def colourize(self, filePath):
        '''
        给黑白图片上色 https://ai.baidu.com/ai-doc/IMAGEPROCESS/Bk3bclns3
        :param img_path: 黑白图片的位置
        :param output_path: 上色后图片的保存位置
        :return:
                {
                    "log_id": "6876747463538438254",
                    "image": "处理后图片的Base64编码"
                }
        '''
        file_content = self.get_file_content(filePath)
        res_image = self.CLIENT.colourize(file_content)
        return res_image



# -*- coding: UTF-8 -*-
'''
@Author  ：程序员晚枫，B站/抖音/微博/小红书/公众号
@WeChat     ：CoderWanFeng
@Blog      ：www.python-office.com
@Date    ：2023/1/22 15:23
@Description     ：
'''

from pobaidu.core.ImageProcess import ImageProcess
import requests


def get_ImageProcess(configPath):
    ip = ImageProcess()
    ip.set_config(configPath)
    return ip


def colourize(img_path, output_path=r'./', configPath=None):
    """
    黑白照片上色
    :param img_path:
    :param output_path:
    :param configPath:
    :return:
    """
    ip = get_ImageProcess(configPath)
    file_data = ip.colourize(img_path)
    ip.save_file_content(filePath=output_path, fileData=file_data)


def selfieAnime(filePath, output_path=r'./', configPath=None):
    ip = get_ImageProcess(configPath)
    file_data = ip.selfieAnime(filePath)
    ip.save_file_content(filePath=output_path, fileData=file_data)


def face_merge():
    '''
    人脸融合
    '''

    request_url = "https://aip.baidubce.com/rest/2.0/face/v1/merge"

    params = "{\"image_template\":{\"image\":\"sfasq35sadvsvqwr5q...\",\"image_type\":\"BASE64\",\"quality_control\":\"NONE\"},\"image_target\":{\"image\":\"sfasq35sadvsvqwr5q...\",\"image_type\":\"BASE64\",\"quality_control\":\"NONE\"}}"
    access_token = '[调用鉴权接口获取的token]'
    request_url = request_url + "?access_token=" + access_token
    headers = {'content-type': 'application/json'}
    response = requests.post(request_url, data=params, headers=headers)
    if response:
        print(response.json())



import unittest

from pobaidu.core.ImageProcess import ImageProcess

from pobaidu.api.imageprocess import selfie_anime


class TestBaidu(unittest.TestCase):
    def test_selfie_anime(self):
        selfie_anime(img_path=r'file/img_cartoon.jpg')

    def test_colourize(self):
        ip = ImageProcess()
        res = ip.colourize(img_path=r'file/girl.jpg',output_path=r'./output/res.jpg')
        print(res)



import toml

from pobaidu.lib.CONST import DEFAULT_CONFIG_PATH_NAME


# def get_toml(toml_path):
#     """
#     解析配置文件
#     :param toml_path: 存放配置文件的位置和名称，在py文件的同级路径下
#     :return: 加载后的信息
#     """
#     try:
#         config_info = toml.load(toml_path)
#         print('配置文件【baidu-config.toml】读取成功')
#         return config_info
#     except:
#         print('配置文件【baidu-config.toml】读取失败，请查看视频，进行配置：https://www.bilibili.com/video/BV1Zv4y1r7zq')


def get_error_info(error_info):
    """
    检查接口返回数据是否错误
    :param error_info: 调用接口的返回数据
    :return:
    """
    error_url = 'http://python4office.cn/pobaidu/pobaidu-error/'
    if error_info.get('error_code', False):
        return f"接口调用错误，错误信息是{error_info}，原因和解决方法请查看官方文档：{error_url}"
    return False



# -*- coding: UTF-8 -*-
'''
@Author  ：程序员晚枫，B站/抖音/微博/小红书/公众号
@WeChat     ：CoderWanFeng
@Blog      ：www.python-office.com
@Date    ：2023/2/2 23:39 
@Description     ：
'''

import base64

import requests
import json

def face_merge(client_id, client_secret, base_img='', face_img='', output_path=r'./output/face_merge.jpg'):
    url = f"https://aip.baidubce.com/oauth/2.0/token?client_id={client_id}&client_secret={client_secret}&grant_type=client_credentials"

    payload = ""
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }

    token_response = requests.request("POST", url, headers=headers, data=payload)

    print(token_response.text)

    res = json.loads(token_response.text)
    access_token = res['access_token']
    url = f"https://aip.baidubce.com/rest/2.0/face/v1/merge?access_token={access_token}"
    with open(base_img, 'rb') as fp:
        base_img = base64.b64encode(fp.read()).decode()
    with open(face_img, 'rb') as fp:
        face_img = base64.b64encode(fp.read()).decode()
    print(base_img)
    payload = json.dumps({
        "image_target": {
            "image": base_img,
            "image_type": "BASE64",
            "quality_control": "NONE"
        },
        "image_template": {
            "image": face_img,
            "image_type": "BASE64"
        },
        "merge_degree": "HIGH"
    })
    headers = {
        'Content-Type': 'application/json'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print(json.loads(response.text)['result']['merge_image'])
    res_img = json.loads(response.text)['result']['merge_image']
    # 解码图片
    imgdata = base64.b64decode(res_img)
    # 将图片保存为文件
    with open(output_path, 'wb') as f:
        f.write(imgdata)




# -*- coding: UTF-8 -*-
'''
@Author  ：程序员晚枫，B站/抖音/微博/小红书/公众号
@WeChat     ：CoderWanFeng
@Blog      ：www.python-office.com
@Date    ：2023/1/22 18:26 
@Description     ：
'''

DEFAULT_CONFIG_PATH = r'./'
DEFAULT_CONFIG_NAME = r'baidu-config.toml'

DEFAULT_CONFIG_PATH_NAME = DEFAULT_CONFIG_PATH + DEFAULT_CONFIG_NAME



# -*- coding: UTF-8 -*-
'''
@Author  ：程序员晚枫，B站/抖音/微博/小红书/公众号
@WeChat     ：CoderWanFeng
@Blog      ：www.python-office.com
@Date    ：2023/1/22 18:45 
@Description     ：
'''
import toml
from pobaidu.lib.CONST import DEFAULT_CONFIG_PATH_NAME


class pobaiduConfig():
    def __init__(self):
        self.config_info = None

    def get_config(self, configPath):
        """
        解析配置文件
        :param toml_path: 存放配置文件的位置和名称，在py文件的同级路径下
        :return: 加载后的信息
        """
        try:
            if configPath == None:
                self.config_info = toml.load(DEFAULT_CONFIG_PATH_NAME)
            else:
                self.config_info = toml.load(configPath)
            print('配置文件【baidu-config.toml】读取成功')
            return self.config_info
        except:
            print('配置文件【baidu-config.toml】读取失败，请查看视频，进行配置：https://www.bilibili.com/video/BV1Zv4y1r7zq')
